package server

import (
	"asapo_common/utils"
	"github.com/stretchr/testify/assert"
	"io/ioutil"
	"net/http"
	"net/http/httptest"
	"os"
	"path/filepath"
	"strings"
	"testing"
)

func prepareToken(beamtime_id string) string{
	auth = utils.NewHMACAuth("secret")
	token, _ := auth.GenerateToken(&beamtime_id)
	return token
}


type request struct {
	path    string
	cmd     string
	answer  int
	message string
}

func allowBeamlines(beamlines []beamtimeInfo) {
	settings.AlwaysAllowedBeamtimes=beamlines
}


func containsMatcher(substr string) func(str string) bool {
	return func(str string) bool { return strings.Contains(str, substr) }
}

func makeRequest(request authorizationRequest) string {
	buf, _ := utils.MapToJson(request)
	return string(buf)
}

func doAuthorizeRequest(path string,buf string) *httptest.ResponseRecorder {
	mux := utils.NewRouter(listRoutes)
	req, _ := http.NewRequest("POST", path, strings.NewReader(buf))
	w := httptest.NewRecorder()
	mux.ServeHTTP(w, req)
	return w
}


var credTests = [] struct {
	request string
	cred SourceCredentials
	message string
} {
	{"asapo_test%%", SourceCredentials{"asapo_test","detector",""},"default stream and no token"},
	{"asapo_test%%token", SourceCredentials{"asapo_test","detector","token"},"default stream"},
	{"asapo_test%stream%", SourceCredentials{"asapo_test","stream",""},"no token"},
	{"asapo_test%stream%token", SourceCredentials{"asapo_test","stream","token"},"all set"},
}

func TestSplitCreds(t *testing.T) {

	for _, test := range credTests {
		request :=  authorizationRequest{test.request,"host"}
		creds,err := getSourceCredentials(request)
		assert.Nil(t,err)
		assert.Equal(t,creds,test.cred,test.message)
	}
}

func TestAuthorizeDefaultOK(t *testing.T) {
	allowBeamlines([]beamtimeInfo{{"asapo_test","beamline","","2019","tf"}})
	request :=  makeRequest(authorizationRequest{"asapo_test%%","host"})
	w := doAuthorizeRequest("/authorize",request)

	body, _ := ioutil.ReadAll(w.Body)

	assert.Contains(t, string(body), "asapo_test", "")
	assert.Contains(t, string(body), "beamline", "")
	assert.Contains(t, string(body), "detector", "")

	assert.Equal(t, http.StatusOK, w.Code, "")
}

var authTests = [] struct {
	beamtime_id string
	stream string
	token string
	status int
	message string
}{
	{"test","stream", prepareToken("test"),http.StatusOK,"user stream with correct token"},
	{"test1","stream", prepareToken("test1"),http.StatusUnauthorized,"correct token, beamtime not found"},
	{"test","stream", prepareToken("wrong"),http.StatusUnauthorized,"user stream with wrong token"},
	{"test","detector_aaa", prepareToken("test"),http.StatusUnauthorized,"detector stream with correct token and wroung source"},
}
func TestAuthorizeWithToken(t *testing.T) {
	allowBeamlines([]beamtimeInfo{})
	settings.RootBeamtimesFolder ="."
	os.MkdirAll(filepath.Clean("tf/gpfs/bl1/2019/data/test"), os.ModePerm)
	defer 	os.RemoveAll("tf")

	for _, test := range authTests {
		request :=  makeRequest(authorizationRequest{test.beamtime_id+"%"+test.stream+"%"+test.token,"host"})
		w := doAuthorizeRequest("/authorize",request)

		body, _ := ioutil.ReadAll(w.Body)
		if test.status==http.StatusOK {
			assert.Contains(t, string(body), test.beamtime_id, "")
			assert.Contains(t, string(body), "bl1", "")
			assert.Contains(t, string(body), "stream", "")
			assert.Contains(t, string(body), "2019", "")
			assert.Contains(t, string(body), "tf", "")
			assert.Contains(t, string(body), test.stream, "")
		}

		assert.Equal(t, test.status, w.Code, test.message)
	}


}



func TestNotAuthorized(t *testing.T) {
	request :=  makeRequest(authorizationRequest{"any_id%%","host"})
	w := doAuthorizeRequest("/authorize",request)
	assert.Equal(t, http.StatusUnauthorized, w.Code, "")
}


func TestAuthorizeWrongRequest(t *testing.T) {
	w := doAuthorizeRequest("/authorize","babla")
	assert.Equal(t, http.StatusBadRequest, w.Code, "")
}


func TestAuthorizeWrongPath(t *testing.T) {
	w := doAuthorizeRequest("/authorized","")
	assert.Equal(t, http.StatusNotFound, w.Code, "")
}

func TestDoNotAuthorizeIfNotInAllowed(t *testing.T) {
	allowBeamlines([]beamtimeInfo{{"test","beamline","","2019","tf"}})

	request :=  authorizationRequest{"asapo_test%%","host"}
	creds,_ := getSourceCredentials(request)
	_,ok := authorize(request,creds)
	assert.Equal(t,false, ok, "")
}

func TestSplitHost(t *testing.T) {
	host := splitHost("127.0.0.1:112")
	assert.Equal(t,"127.0.0.1", host, "")
}


func TestSplitHostNoPort(t *testing.T) {
	host := splitHost("127.0.0.1")
	assert.Equal(t,"127.0.0.1", host, "")
}

func TestGetBeamlineFromIP(t *testing.T) {
	beamline, err := getBeamlineFromIP("127.0.0.1:112")
	assert.NotNil(t,err, "")
	assert.Empty(t,beamline, "")

}

func TestAuthorizeWithFile(t *testing.T) {
	settings.IpBeamlineMappingFolder="."
	settings.RootBeamtimesFolder ="."
	os.MkdirAll(filepath.Clean("tf/gpfs/bl1/2019/data/11003924"), os.ModePerm)


	ioutil.WriteFile("127.0.0.1", []byte("bl1"), 0644)


	request := authorizationRequest{"11003924%%","127.0.0.1"}
	w := doAuthorizeRequest("/authorize",makeRequest(request))

	body, _ := ioutil.ReadAll(w.Body)
	assert.Contains(t, string(body), "11003924", "")
	assert.Contains(t, string(body), "bl1", "")
	assert.Contains(t, string(body), "detector", "")
	assert.Contains(t, string(body), "2019", "")
	assert.Contains(t, string(body), "tf", "")
	assert.Equal(t, http.StatusOK, w.Code, "")

	request = authorizationRequest{"wrong%%","127.0.0.1"}
	w = doAuthorizeRequest("/authorize",makeRequest(request))
	assert.Equal(t, http.StatusUnauthorized, w.Code, "")

	os.Remove("127.0.0.1")
	os.RemoveAll("tf")

}


var extractBtinfoTests = [] struct {
	root string
	fname string
	facility string
	beamline string
	year string
	id string
	ok bool
}{
	{".",filepath.Clean("tf/gpfs/bl1.01/2019/data/123"),"tf", "bl1.01","2019","123",true},
	{filepath.Clean("/blabla/tratartra"),filepath.Clean("tf/gpfs/bl1.01/2019/data/123"),"tf", "bl1.01","2019","123",true},
	{".",filepath.Clean("tf/gpfs/common/2019/data/123"),"tf", "bl1.01","2019","123",false},
	{".",filepath.Clean("tf/gpfs/BeamtimeUsers/2019/data/123"),"tf", "bl1.01","2019","123",false},
	{".",filepath.Clean("tf/gpfs/state/2019/data/123"),"tf", "bl1.01","2019","123",false},
	{".",filepath.Clean("tf/gpfs/support/2019/data/123"),"tf", "bl1.01","2019","123",false},
	{".",filepath.Clean("petra3/gpfs/p01/2019/comissioning/c20180508-000-COM20181"),"petra3", "p01","2019","c20180508-000-COM20181",true},

}
func TestGetBeamtimeInfo(t *testing.T) {
	for _, test := range extractBtinfoTests {
		settings.RootBeamtimesFolder=test.root
		bt,err:=beamtimeInfoFromMatch(test.root+string(filepath.Separator)+test.fname)
		if test.ok {
			assert.Equal(t,bt.Facility,test.facility)
			assert.Equal(t,bt.Beamline,test.beamline)
			assert.Equal(t,bt.Year,test.year)
			assert.Equal(t,bt.BeamtimeId,test.id)
			assert.Equal(t,bt.Facility,test.facility)
			assert.Nil(t,err,"should not be error")
		} else {
			assert.NotNil(t,err,"should be error")
		}
	}

}
